@@ -147,14 +147,27 @@ class PM4Sand(sm.StressDependentSoil):
     def g_mod_at_v_eff_stress(self, sigma_v_eff):  # Override base function since k0 is different
         return self.get_g_mod_at_v_eff_stress(sigma_v_eff)
 
-    def get_g_mod_at_v_eff_stress(self, sigma_v_eff):  # Override base function since k0 is different
-        k0 = 1 - np.sin(self.phi_r)
+    def get_g_mod_at_v_eff_stress(self, sigma_v_eff, k0=None):  # Override base function since k0 is different
+        if k0 is None:
+            k0 = self.poissons_ratio / (1 - self.poissons_ratio)
         return self.g0_mod * self.p_atm * (sigma_v_eff * (1 + k0) / 2 / self.p_atm) ** 0.5
 
-    def set_g0_mod_from_g_mod_at_v_eff_stress(self, g_mod, sigma_v_eff):
-        k0 = 1 - np.sin(self.phi_r)
+    def set_g0_mod_from_g_mod_at_v_eff_stress(self, g_mod, sigma_v_eff, k0=None):
+        if k0 is None:
+            k0 = self.poissons_ratio / (1 - self.poissons_ratio)
         self.g0_mod = g_mod / self.p_atm / (sigma_v_eff * (1 + k0) / 2 / self.p_atm) ** 0.5
 
+    def get_peak_angle(self, p):
+        n_b = self.n_b
+        if n_b is None:
+            n_b = 0.5
+        q_bolt = self.q_bolt
+        if q_bolt is None:
+            q_bolt = 10
+        r_bolt = self.r_bolt
+        if r_bolt is None:
+            r_bolt = 1.5
+        return calc_peak_angle_for_pm4sand(self.relative_density, p, p_atm=self.p_atm, phi_cv=self.phi_cv, n_b=n_b, q_bolt=q_bolt, r_bolt=r_bolt)
 
     # def e_critical(self, p):
     #     p = float(p)
@@ -171,6 +184,125 @@ class PM4Sand(sm.StressDependentSoil):
     #         return None
 
 
+def calc_peak_angle_for_pm4sand(d_r, p, p_atm=101.0e3, phi_cv=33.0, n_b=0.5, q_bolt=10, r_bolt=1.5):
+    dr_cs = r_bolt / (q_bolt - np.log(100 * p / p_atm))  # Eq 11
+    ksi_r = dr_cs - d_r  # Eq 10
+    phi_r = np.radians(phi_cv)
+    big_m = 2 * np.sin(phi_r)  # Eq 14
+    if ksi_r < 0:
+        m_b = big_m * np.exp(-n_b * ksi_r)  # Eq 13
+    else:
+        m_b = big_m * np.exp(-n_b / 4 * ksi_r)  # from source code ?
+    return np.degrees(np.arcsin(0.5 * m_b))  # Eq 46
+
+
+class ManzariDafaliasModel(sm.StressDependentSoil):
+    crr_n15 = None
+    m_c = None
+    c_c = None  # c = Me / Mc, the ratio of critical stress ratios in triaxial compression (Mc) and extension (Me)
+    lambda_c = None
+    e_0 = None
+    ksi = None
+    m_yield = None
+    h_0 = None
+    c_h = None
+    n_b = None
+    n_d = None
+    a_o = None
+    z_max = None
+    c_z = None
+    _g0 = None
+    int_scheme = None
+
+    # TODO: add non default inputs here
+    type = "manzaridafalias_model"
+
+    def __init__(self, pw=None, wmd=None, liq_mass_density=None, liq_sg=1, g=9.8, p_atm=101000.0, **kwargs):
+        # Note: pw has deprecated
+        # _gravity = g  # m/s2
+        # if liq_mass_density:
+        #     _liq_mass_density = liq_mass_density  # kg/m3
+        # elif pw is not None and _gravity is not None:
+        #     if pw == 9800 and g == 9.8:
+        #         _liq_mass_density = 1.0e3
+        #     else:
+        #         _liq_mass_density = pw / _gravity
+        # else:
+        #     _liq_mass_density = None
+
+        sm.StressDependentSoil.__init__(self, pw=pw, wmd=wmd, liq_mass_density=liq_mass_density, liq_sg=liq_sg, g=g, **kwargs)
+        self._extra_class_inputs = [
+            "crr_n15",
+            "m_c",
+            "c_c",
+            "lambda_c",
+            "e_0",
+            "ksi",
+            "m_yield",
+            "h_0",
+            "c_h",
+            "n_b",
+            "n_d",
+            "a_o",
+            "z_max",
+            "c_z",
+            "g0",
+            'int_scheme'
+        ]
+        self.p_atm = p_atm
+        self.inputs += self._extra_class_inputs
+
+        if not hasattr(self, "definitions"):
+            self.definitions = OrderedDict()
+        self.definitions["crr_n15"] = ["Cyclic resistance ratio for 15 cycles", "-"]
+        self.definitions["g0"] = ["Normalised shear modulus factor", "-"]
+        self.definitions["p_atm"] = ["Atmospheric pressure", "Pa"]
+
+    def __repr__(self):
+        return "ManzariDafaliasModel Soil model, id=%i, phi=%.1f, Dr=%.2f" % (self.id, self.phi, self.relative_density)
+
+    def __str__(self):
+        return "ManzariDafaliasModel Soil model, id=%i, phi=%.1f, Dr=%.2f" % (self.id, self.phi, self.relative_density)
+
+    def get_peak_angle(self, p):
+        e_cs = self.e_0 - self.lambda_c * (p / self.p_atm) ** self.ksi
+        psi = self.e_curr - e_cs
+        m_b = self.m_c * np.exp(-self.n_b * psi)  # Eq 13
+        
+        return np.degrees(np.arcsin(0.5 * m_b))  # Eq 46
+    
+    def get_crit_angle(self):
+        phi_r = np.arcsin(self.m_c / 2)
+        return np.degrees(phi_r)
+    
+    def set_big_m_from_phi_cv(self, phi_cv):
+        phi_r = np.radians(phi_cv)
+        self.m_c = 2 * np.sin(phi_r)
+
+    @property
+    def g0(self):
+        return self._g0
+
+    @g0.setter
+    def g0(self, g0):
+        # note g0 * (2.97 - e) ** 2 / (1 + e) = g0_mod
+        self._g0 = clean_float(g0)
+        if self.e_curr is not None:
+            self._g0_mod = g0 * (2.97 - self.e_curr) ** 2 / (1 + self.e_curr)
+
+    @property
+    def g0_mod(self):
+        return self._g0_mod
+
+    @g0_mod.setter
+    def g0_mod(self, value):
+        value = clean_float(value)
+        self._g0_mod = value
+        if self.e_curr is None:
+            raise ValueError('must set e_curr before setting g0_mod')
+        self._g0 = value * (1 + self.e_curr) / (2.97 - self.e_curr) ** 2
+
+
 class StressDensityModel(sm.StressDependentSoil):
     _crr_n15 = None
     big_a = None
@@ -285,4 +417,3 @@ class StressDensityModel(sm.StressDependentSoil):
         k0 = 1 - np.sin(self.phi_r)
         p = sigma_v_eff * (1 + k0) / 2
         self.g0_mod = g_mod / self.p_atm / (p / self.p_atm) ** self.a
-
